/****************************************************************************/
/*  File:       Zip.java                                                    */
/*  Author:     F. Georges - fgeorges.org                                   */
/*  Date:       2009-08-03                                                  */
/*  Tags:                                                                   */
/*      Copyright (c) 2009 Florent Georges (see end of file.)               */
/* ------------------------------------------------------------------------ */


package org.expath.saxon;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.Reader;
import java.net.URI;
import java.net.URISyntaxException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;
import javax.xml.transform.Source;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.stream.StreamSource;
import net.sf.saxon.Configuration;
import net.sf.saxon.event.PipelineConfiguration;
import net.sf.saxon.expr.XPathContext;
import net.sf.saxon.om.Axis;
import net.sf.saxon.om.AxisIterator;
import net.sf.saxon.om.NamePool;
import net.sf.saxon.om.NodeInfo;
import net.sf.saxon.trans.XPathException;
import net.sf.saxon.tree.TreeBuilder;
import net.sf.saxon.type.Type;
import net.sf.saxon.value.Base64BinaryValue;
import net.sf.saxon.value.Whitespace;
import org.ccil.cowan.tagsoup.Parser;
import org.expath.zip.Entry;
import org.expath.zip.ZipConstants;
import org.expath.zip.ZipTree;
import org.expath.zip.ZipTree.ZipNode;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * EXPath ZIP module implementation for Saxon.
 *
 * @author Florent Georges
 * @date   2009-08-03
 */
public class Zip
{
    public Zip(String base_uri)
    {
        myBaseUri = base_uri;
    }

    public static NodeInfo xmlEntry(XPathContext ctxt, String href, String path)
            throws XPathException
    {
        // FIXME: How to access the base URI from here?  I think this is not
        // possible, so I pass null for now...
        Zip zip = new Zip(null);
        return zip.doXmlEntry(ctxt, href, path);
    }

    public static NodeInfo htmlEntry(XPathContext ctxt, String href, String path)
            throws XPathException
    {
        // FIXME: How to access the base URI from here?  I think this is not
        // possible, so I pass null for now...
        Zip zip = new Zip(null);
        return zip.doHtmlEntry(ctxt, href, path);
    }

    public static String textEntry(XPathContext ctxt, String href, String path)
            throws XPathException
    {
        // FIXME: How to access the base URI from here?  I think this is not
        // possible, so I pass null for now...
        Zip zip = new Zip(null);
        return zip.doTextEntry(ctxt, href, path);
    }

    public static Base64BinaryValue binaryEntry(XPathContext ctxt, String href, String path)
            throws XPathException
    {
        // FIXME: How to access the base URI from here?  I think this is not
        // possible, so I pass null for now...
        Zip zip = new Zip(null);
        return zip.doBinaryEntry(ctxt, href, path);
    }

    public static NodeInfo entries(XPathContext ctxt, String href)
            throws XPathException
    {
        // FIXME: How to access the base URI from here?  I think this is not
        // possible, so I pass null for now...
        Zip zip = new Zip(null);
        return zip.doEntries(ctxt, href);
    }

    public static void zipFile(XPathContext ctxt, NodeInfo struct)
            throws XPathException
    {
        Zip zip = new Zip(null);
        zip.doZipFile(ctxt, struct);
    }

    public static void updateEntries(XPathContext ctxt, NodeInfo struct, String dest)
            throws XPathException
    {
        // FIXME: How to access the base URI from here?  I think this is not
        // possible, so I pass null for now...
        Zip zip = new Zip(null);
        zip.doUpdateEntries(ctxt, struct, dest);
    }

    public NodeInfo doXmlEntry(XPathContext ctxt, String href, String path)
            throws XPathException
    {
        return mlEntry(ctxt, href, path, false);
    }

    public NodeInfo doHtmlEntry(XPathContext ctxt, String href, String path)
            throws XPathException
    {
        return mlEntry(ctxt, href, path, true);
    }

    private NodeInfo mlEntry(XPathContext ctxt, String href, String path, boolean html)
            throws XPathException
    {
        // TODO: ...
        String charset = "utf-8";
        String sys_id = "TODO-find-a-useful-systemId";
        File input = fileFromURI(href, myBaseUri);
        try {
            ZipFile zip = new ZipFile(input);
            ZipEntry entry = zip.getEntry(path);
            InputStream in = zip.getInputStream(entry);
            Reader reader = new InputStreamReader(in, charset);
            Source src = null;
            if ( html ) {
                Parser parser = new Parser();
                parser.setFeature("http://xml.org/sax/features/namespace-prefixes", true);
                src = new SAXSource(parser, new InputSource(reader));
                src.setSystemId(sys_id);
            }
            else {
                src = new StreamSource(reader, sys_id);
            }
            return ctxt.getConfiguration().buildDocument(src);
        }
        catch ( ZipException ex ) {
            throw new XPathException("Wrong ZIP file: " + input.getAbsolutePath(), ex);
        }
        catch ( IOException ex ) {
            throw new XPathException("Error reading ZIP file: " + input.getAbsolutePath(), ex);
        }
        catch ( SAXException ex ) {
            throw new XPathException("Error parsing HTML entry: " + input.getAbsolutePath() + "!" + path, ex);
        }
    }

    // TODO: What if the entry does not exist?
    public String doTextEntry(XPathContext ctxt, String href, String path)
            throws XPathException
    {
        // TODO: ...
        String charset = "utf-8";
        try {
            ZipFile zip = new ZipFile(fileFromURI(href, myBaseUri));
            ZipEntry entry = zip.getEntry(path);
            InputStream in = zip.getInputStream(entry);
            Reader reader = new InputStreamReader(in, charset);
            StringBuilder buf = new StringBuilder();
            char[] ch = new char[4096];
            int i = -1;
            while ( (i = reader.read(ch)) > 0 ) {
                buf.append(ch, 0, i);
            }
            return buf.toString();
        }
        catch ( ZipException ex ) {
            throw new XPathException("Wrong ZIP file", ex);
        }
        catch ( IOException ex ) {
            throw new XPathException("Error reading ZIP file", ex);
        }
    }

    public Base64BinaryValue doBinaryEntry(XPathContext ctxt, String href, String path)
            throws XPathException
    {
        try {
            ZipFile zip = new ZipFile(fileFromURI(href, myBaseUri));
            ZipEntry entry = zip.getEntry(path);
            InputStream in = zip.getInputStream(entry);
            ByteArrayOutputStream buf = new ByteArrayOutputStream();
            byte[] b = new byte[4096];
            int i = -1;
            while ( (i = in.read(b)) > 0 ) {
                buf.write(b, 0, i);
            }
            return new Base64BinaryValue(buf.toByteArray());
        }
        catch ( ZipException ex ) {
            throw new XPathException("Wrong ZIP file", ex);
        }
        catch ( IOException ex ) {
            throw new XPathException("Error reading ZIP file", ex);
        }
    }

    public NodeInfo doEntries(XPathContext ctxt, String href)
            throws XPathException
    {
        try {
            ZipFile zip = new ZipFile(fileFromURI(href, myBaseUri));
            ZipTree tree = new ZipTree(zip);
            // the config and pipeline objects
            Configuration config = ctxt.getConfiguration();
            PipelineConfiguration pipe = new PipelineConfiguration();
            pipe.setConfiguration(config);
            // the builder and name pool
            TreeBuilder b = new TreeBuilder();
            b.setPipelineConfiguration(pipe);
            NamePool pool = config.getNamePool();
            // start...
            b.open();
            // root element
            final String prefix = ZipConstants.ZIP_NS_PREFIX;
            final String uri = ZipConstants.ZIP_NS_URI;
            int root_name = pool.allocate(prefix, uri, "file");
            b.startElement(root_name, -1, 0, 0);
            int code_name = pool.allocate("", "", "href");
            b.attribute(code_name, -1, href, 0, 0);
            b.startContent();
            for ( Map.Entry<String, ZipNode> child : tree.getRoot().getNodes().entrySet() ) {
                outputTree(child.getValue(), b, pool);
            }
            b.endElement();
            b.close();
            return b.getCurrentRoot();
        }
        catch ( ZipException ex ) {
            throw new XPathException("Wrong ZIP file", ex);
        }
        catch ( IOException ex ) {
            throw new XPathException("Error reading ZIP file", ex);
        }
    }

    /**
     * Create a new ZIP file.
     *
     * Take as parameter a zip:file element node.  If the element contains an
     * attribute {@code href}, it must be a URI with file: scheme, and the file
     * is written on this URI (and this function returns null).  If not, the
     * file is returned as a base64 item.
     */
    public Base64BinaryValue doZipFile(XPathContext ctxt, NodeInfo struct)
            throws XPathException
    {
        Output out = new Output(struct);
//        File zip_file = analyseHref(struct);
//        ZipOutputStream zip = openZipFileOutput(zip_file);
        ZipOutputStream zip = out.getZipStream();
        // walk the elements
        AxisIterator childs = struct.iterateAxis(Axis.CHILD);
        NodeInfo child;
        while ( (child = (NodeInfo) childs.next()) != null ) {
            if ( isEntry(child) ) {
                handleEntry(child, zip, new Entry.Path());
            }
        }
        closeStream(zip);
        return out.getBase64();
    }

    // TODO: Allow to remove existing entries too.
    // TODO: Accept another param, the name of the output file.  Href could refer
    // to the source, and we could have a different target (so it is easy to have
    // an ODF pattern and just update a few files within it, but to another
    // destination, so the pattern file is never modified.)
    public Base64BinaryValue doUpdateEntries(XPathContext ctxt, NodeInfo struct, String dest)
            throws XPathException
    {
        try {
            byte[] bytes = new byte[4096];
            Map<String, Entry> entries = new HashMap<String, Entry>();
            accumulateEntries(struct, entries, new Entry.Path());
            File zip_file = analyseHref(struct);
            ZipInputStream zip = openZipFileInput(zip_file);
            File out_file = dest == null ? null : fileFromURI(dest, myBaseUri);
            Output output = new Output(out_file);
            ZipOutputStream out = output.getZipStream();
            for ( ZipEntry e = zip.getNextEntry(); e != null; e = zip.getNextEntry() ) {
                Entry entry = entries.get(e.getName());
                if ( entry == null ) {
                    out.putNextEntry(new ZipEntry(e.getName()));
                    // copy from zip to out
                    int l = -1;
                    while ( (l = zip.read(bytes)) != -1 ) {
                        out.write(bytes, 0, l);
                    }
                    out.closeEntry();
                }
                else {
                    entry.serialize(out);
                    entries.remove(entry.getPath());
                }
            }
            zip.close();
            // remaining entries, to add (that were not in the ZIP initially)
            for ( Entry entry : entries.values() ) {
                entry.serialize(out);
            }
            out.close();
            return output.getBase64();
        }
        catch ( IOException ex ) {
            throw new XPathException("Error opening temporary file", ex);
        }
    }

    private File analyseHref(NodeInfo zip_elem)
            throws XPathException
    {
        AxisIterator attrs = zip_elem.iterateAxis(Axis.ATTRIBUTE);
        NodeInfo a;
        while ( (a = (NodeInfo) attrs.next()) != null ) {
            String local = a.getLocalPart();
            if ( !"".equals(a.getURI()) ) {
                // ignore namespace qualified attributes
            }
            else if ( "href".equals(local) ) {
                return fileFromURI(a.getStringValue(), zip_elem.getBaseURI());
            }
            else {
                throw new XPathException("Unknown attribute zip:file/@" + local);
            }
        }
        throw new XPathException("required @href has not been set on zip:file");
    }

    /**
     * Represent an output ZIP.
     * 
     * It is consrtucted from a zip:file element.  If it has a href attribute,
     * it must be a target file to write to.  If not, the output must be
     * returned as a base64 item.
     */
    private class Output
    {
        public Output(File f)
        {
            myFile = f;
        }

        public Output(NodeInfo zip_elem)
            throws XPathException
        {
            AxisIterator attrs = zip_elem.iterateAxis(Axis.ATTRIBUTE);
            NodeInfo a;
            while ( (a = (NodeInfo) attrs.next()) != null ) {
                String local = a.getLocalPart();
                if ( !"".equals(a.getURI()) ) {
                    // ignore namespace qualified attributes
                }
                else if ( "href".equals(local) ) {
                    myFile = fileFromURI(a.getStringValue(), zip_elem.getBaseURI());
                }
                else {
                    throw new XPathException("Unknown attribute zip:file/@" + local);
                }
            }
        }

        public ZipOutputStream getZipStream()
                throws XPathException
        {
            if ( myZip == null ) {
                if ( myFile == null ) {
                    myBuffer = new ByteArrayOutputStream();
                    myZip = new ZipOutputStream(myBuffer);
                }
                else {
                    myZip = openZipFileOutput(myFile);
                }
            }
            return myZip;
        }

        public Base64BinaryValue getBase64()
        {
            if ( myBuffer == null ) {
                return null;
            }
            else {
                return new Base64BinaryValue(myBuffer.toByteArray());
            }
        }

        private File myFile = null;
        private ByteArrayOutputStream myBuffer = null;
        private ZipOutputStream myZip = null;
    }

    /**
     * Resolve {@code @href} against the base URI, if needed, and return a {@code File}.
     */
    private File fileFromURI(String uri, String base)
            throws XPathException
    {
        try {
            // parse the URI
            URI result = new URI(uri);
            // resolve against base URI if not absolute yet
            if ( ! result.isAbsolute() ) {
                if ( base == null ) {
                    throw new XPathException("FIXME: Impossible to get the base URI");
                }
                URI base_uri = new URI(base);
                result = base_uri.resolve(result);
            }
            // return it as a File object
            return new File(result);
        }
        catch ( URISyntaxException ex ) {
            throw new XPathException("Invalid URI: " + uri, ex);
        }
    }

    private ZipOutputStream openZipFileOutput(File zip)
            throws XPathException
    {
        ZipOutputStream out = null;
        try {
            out = new ZipOutputStream(new FileOutputStream(zip));
        }
        catch ( FileNotFoundException ex ) {
            throw new XPathException("Error opening ZIP file for writing", ex);
        }
        return out;
    }

    private ZipInputStream openZipFileInput(File zip)
            throws XPathException
    {
        ZipInputStream in = null;
        try {
            in = new ZipInputStream(new FileInputStream(zip));
        }
        catch ( FileNotFoundException ex ) {
            throw new XPathException("Error opening ZIP file for reading", ex);
        }
        return in;
    }

    private void closeStream(OutputStream out)
            throws XPathException
    {
        try {
            out.close();
        }
        catch ( IOException ex ) {
            throw new XPathException("Error closing zip file", ex);
        }
    }

    // if unknown child of zip:file, throw an error, if not then return true if
    // it is an entry element, or false if it is an element to ignore (in another
    // namespace for instance
    private boolean isEntry(NodeInfo node)
            throws XPathException
    {
        String local = node.getLocalPart();
        int kind = node.getNodeKind();
        if ( kind == Type.TEXT ) {
            if ( ! Whitespace.isWhite(node.getStringValueCS()) ) {
                throw new XPathException("Non-whitespace text nodes are not allowed");
            }
            return false;
        }
        else if ( kind == Type.COMMENT || kind == Type.PROCESSING_INSTRUCTION ) {
            // ignore comment and PI nodes
            return false;
        }
        else if ( kind != Type.ELEMENT ) {
            // namespace, document and attribute nodes cannot be no the child axis
            throw new XPathException("Not an element: could not happen");
        }
        else if ( "".equals(node.getURI()) ) {
            // elements in no namespace are an error
            throw new XPathException("Element in no namespace: " + local);
        }
        else if ( ! ZipConstants.ZIP_NS_URI.equals(node.getURI()) ) {
            // ignore elements in other namespaces
            return false;
        }
        else if ( "entry".equals(local) || "dir".equals(local) ) {
            return true;
        }
        else {
            throw new XPathException("Unknown element: " + local);
        }
    }

    private void accumulateEntries(NodeInfo node, Map<String, Entry> map, Entry.Path path)
            throws XPathException
    {
        // walk the elements
        AxisIterator childs = node.iterateAxis(Axis.CHILD);
        NodeInfo child;
        while ( (child = (NodeInfo) childs.next()) != null ) {
            if ( isEntry(child) ) {
                Entry e = Entry.makeEntry(path, child);  // make a path.push()
                map.put(e.getPath(), e);
                // if entry is a dir, recurse
                if ( e.isDir() ) {
                    accumulateEntries(child, map, path);
                }
                path.pop();
            }
        }
    }

    private void outputTree(ZipNode node, TreeBuilder b, NamePool pool)
            throws XPathException
    {
        final String prefix = ZipConstants.ZIP_NS_PREFIX;
        final String uri = ZipConstants.ZIP_NS_URI;
        int dir_name   = pool.allocate(prefix, uri, "dir");
        int entry_name = pool.allocate(prefix, uri, "entry");
        int name_name  = pool.allocate("", "", "name");
        int comm_name  = pool.allocate("", "", "comment");
        int size_name  = pool.allocate("", "", "size");
        int time_name  = pool.allocate("", "", "time");

        ZipEntry entry = node.getEntry();
        b.startElement(node.isDirectory() ? dir_name : entry_name, -1, 0, 0);
        b.attribute(name_name, -1, node.getName(), 0, 0);
        if ( entry != null ) {
            String comment = entry.getComment();
            if ( comment != null ) {
                b.attribute(comm_name, -1, comment, 0, 0);
            }
            if ( ! node.isDirectory() ) {
                long size = entry.getSize();
                if ( size != -1 ) {
                    b.attribute(size_name, -1, Long.toString(size), 0, 0);
                }
            }
            long time = entry.getTime();
            if ( time != -1 ) {
                Date d = new Date(time);
                DateFormat fmt = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
                b.attribute(time_name, -1, fmt.format(d), 0, 0);
            }
        }
        b.startContent();
        if ( node.getNodes() != null ) {
            for ( Map.Entry<String, ZipNode> child : node.getNodes().entrySet() ) {
                outputTree(child.getValue(), b, pool);
            }
        }
        b.endElement();
    }

    private void handleEntry(NodeInfo entry, ZipOutputStream zip, Entry.Path path)
            throws XPathException
    {
        Entry e = Entry.makeEntry(path, entry); // make path.push()
        e.serialize(zip);
        if ( e.isDir() ) {
            // walk the elements if a directory
            AxisIterator childs = entry.iterateAxis(Axis.CHILD);
            NodeInfo child;
            while ( (child = (NodeInfo) childs.next()) != null ) {
                if ( isEntry(child) ) {
                    handleEntry(child, zip, path);
                }
            }
        }
        path.pop();
    }

    private String myBaseUri;
}


/* ------------------------------------------------------------------------ */
/*  DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS COMMENT.               */
/*                                                                          */
/*  The contents of this file are subject to the Mozilla Public License     */
/*  Version 1.0 (the "License"); you may not use this file except in        */
/*  compliance with the License. You may obtain a copy of the License at    */
/*  http://www.mozilla.org/MPL/.                                            */
/*                                                                          */
/*  Software distributed under the License is distributed on an "AS IS"     */
/*  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See    */
/*  the License for the specific language governing rights and limitations  */
/*  under the License.                                                      */
/*                                                                          */
/*  The Original Code is: all this file.                                    */
/*                                                                          */
/*  The Initial Developer of the Original Code is Florent Georges.          */
/*                                                                          */
/*  Contributor(s): none.                                                   */
/* ------------------------------------------------------------------------ */
