package org.expath.zip;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URI;
import java.util.Properties;
import java.util.zip.CRC32;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;
import javax.xml.transform.OutputKeys;
import net.sf.saxon.Configuration;
import net.sf.saxon.om.Axis;
import net.sf.saxon.om.AxisIterator;
import net.sf.saxon.om.NodeInfo;
import net.sf.saxon.query.QueryResult;
import net.sf.saxon.trans.XPathException;
import net.sf.saxon.value.Base64BinaryValue;
import net.sf.saxon.value.HexBinaryValue;
import org.expath.zip.Entry.Path;

/**
 * ...
 *
 * @author Florent Georges
 * @date   2009-12-21
 */
public class FileEntry
        extends Entry
{
    public FileEntry(Serialization serial, Boolean compress, URI src, Path path, NodeInfo entry)
    {
        super(path.getPath(false));
        boolean c = compress == null ? true : compress;
        myCompress = c ? ZipEntry.DEFLATED : ZipEntry.STORED;
        mySerial = serial;
        mySrc = src;
        myNode = entry;
    }

    public boolean isDir()
    {
        return false;
    }

    @Override
    protected void doSerialize(ZipOutputStream out, ZipEntry entry)
            throws XPathException, IOException
    {
        // the compression method
        entry.setMethod(myCompress);
        // for stored (uncompressed) entries, we need to set the size and CRC,
        // so wee need to buffer the output
        OutputStream first_out = out;
        ByteArrayOutputStream byte_out = null;
        if ( myCompress == ZipEntry.STORED ) {
            byte_out = new ByteArrayOutputStream();
            first_out = byte_out;
        }
        else {
            out.putNextEntry(entry);
        }
        // if no @src
        if ( mySrc == null ) {
            AxisIterator childs = myNode.iterateAxis(Axis.CHILD);
            if ( "base64".equals(mySerial.getMethod()) ) {
                Base64BinaryValue val = new Base64BinaryValue(childs.getStringValue());
                first_out.write(val.getBinaryValue());
            }
            else if ( "hex".equals(mySerial.getMethod()) ) {
                HexBinaryValue val = new HexBinaryValue(childs.getStringValue());
                first_out.write(val.getBinaryValue());
            }
            else {
                Properties    options = mySerial.getOutputOptions();
                Configuration config  = myNode.getConfiguration();
                QueryResult.serializeSequence(childs, config, first_out, options);
            }
        }
        // if @src
        else {
            // TODO: For now, support only local files.  Should be extended to
            // support other schemes as well (at least HTTP and HTTPS.)
            File f = new File(mySrc);
            InputStream in = new FileInputStream(f);
            byte[] buf = new byte[4096];
            int l = -1;
            while ( (l = in.read(buf)) > 0 ) {
                first_out.write(buf, 0, l);
            }
            in.close();
        }
        // if we buffered the output, no actually write it
        if ( byte_out != null ) {
            byte[] bytes = byte_out.toByteArray();
            // set the size
            entry.setSize(bytes.length);
            entry.setCompressedSize(bytes.length);
            // compute the CRC 32
            CRC32 crc = new CRC32();
            crc.update(bytes);
            entry.setCrc(crc.getValue());
            // actually write to the final destination
            out.putNextEntry(entry);
            out.write(bytes);
        }
    }

    private int myCompress;
    private Serialization mySerial;
    private URI mySrc;
    private NodeInfo myNode;
}
